#!/usr/bin/env perl
use warnings;
use strict;
use Getopt::Std;

=pod

=head1 NAME

B<max_dir_depth> -- The Directory Depth Measurement Tool

=head1 SYNOPSIS

B<max_dir_depth> [B<-fud>] I<start_directory>

=head1 DESCRIPTION

The program outputs the maximum directory depth of a file hierarchy (by default), a list of folders
with this directory depth (the 'B<-f>' option) and a list of folders which we can't process for some
reason (the 'B<-u>' option).

The I<start_directory> argument is a folder from which the program measures the directory depth.
The directory depth of the I<start_directory> is of '0'. Directories that are folded in the 
I<start_directory> have the directory depth of '1'. And so on. 

=head1 OPTIONS

=over 4

=item -f               

output a list of folders with the maximum directory depth

=item -u

output a list of unprocessed folders

=item -d

set debug output on

=back

=head1 AUTHOR

Bisyarin Artyom (L<zool@zool.in.ua>)

=cut

###  Getopt::Std
$main::VERSION = 0.2;

$Getopt::Std::STANDARD_HELP_VERSION = 1;

sub main::VERSION_MESSAGE() {
    my ($fh, $name, $version, $switches_string) = @_;
    print "max_dir_depth $main::VERSION - The Directory Depth Measurement Tool\n" . 
          "Copyright (C) 2007, 2009 Bisyarin Artyom (zool\@zool.in.ua)\n\n";
}

sub main::HELP_MESSAGE() {
    print "Usage: $0 [-fud] start_directory\n\n".
          "Outputs the maximum directory depth.\n" .
          "The start_directory argument is a folder from which the program measures the directory depth.\n\n" .
          "Options:\n" .
          "-f\t\t output a list of folders with the maximum directory depth\n" .
          "-u\t\t output a list of unprocessed folders\n" .
          "-d\t\t set debug output on\n" .
          "\n";
}
###

###  
my %opts;
getopts ('fud', \%opts);
###

exec ("$0 --help") and exit if ($ARGV[0] eq '');

&trace ("debug output is on") if ($opts{'d'});
&trace ("list of folders is on")if ($opts{'f'});
&trace ("list of unprocessed folders is on") if ($opts{'u'});

my $start_dir = $ARGV[0];
&trace ("$start_dir is the start directory...");

die "Error: $start_dir is not exists" unless (-e $start_dir);
die "Error: $start_dir is not a directory" unless (-d $start_dir);

my ($returned_max_depth, $dirs_ref, $unproc_dirs_ref) = &get_max_dir_depth($start_dir, $opts{'f'}, $opts{'u'});

if ($returned_max_depth>=0) {
    print "Maximum directory depth: $returned_max_depth\n";

    if ($opts{'f'}) {
        print "\n" . (scalar @{$dirs_ref}==1?"Folder":"Folders") . " with this directory depth:\n";
        {
            local $,="\n";
            print @{$dirs_ref},'';
        }
    }



    if ($opts{'u'} && $#{$unproc_dirs_ref}>=0) {
        print "\n" . "There are folders which we can't process for some reason. Here they are:\n";
        foreach (@{$unproc_dirs_ref}) {
            print $_->{'dir'}, "\t", $_->{'reason'}, "\n";
        }
    }

}



=begin comment
get_max_dir_depth ($path)
     .
 .
 - trace('mesage')    ,
 ,   - .

$path                      -       .
$dirs_list_needed          -        .
$unproc_dirs_list_needed   -        

 :
  -    
  -
                 $dirs_list_needed  :
                 -         .
                 $dirs_list_needed :
                 - 
  - 
                 $unproc_dirs_list_needed  :
                 -    .     :
                   {'dir'=>'','reason'=>''}
                   ..        .
                 $unproc_dirs_list_needed :
                 - 
=end comment

=cut

BEGIN {
my $curr_depth = 0;
my $max_depth = 0;
my @dirs;
my @unproc_dirs;

sub get_max_dir_depth () {
    my $path = shift;
    my $dirs_list_needed = shift;
    my $unproc_dirs_list_needed = shift;

    &trace ("=== START OF PROCESSING $path DIRECTORY ===");
    &trace ("path is $path...");
    die "$path is not a directory " unless (-d $path);
    &trace ("$path is a directory...");

    &trace ("Current directory depth is $curr_depth");    

    if ($curr_depth > $max_depth) { 
        $max_depth = $curr_depth;
        &trace ("new maximum directory depth is $max_depth...");
        
        # FOR $dirs_list_needed ONLY
        #       
        # ---    
        @dirs = () if ($dirs_list_needed);
    }

    # FOR $dirs_list_needed ONLY
    if ($curr_depth == $max_depth && $dirs_list_needed) {
        #        
        #   ---     
        push @dirs, $path;
        &trace ("$path has the maximum directory depth at the moment...");
        &trace ('we have ' . scalar @dirs . ' ' . (scalar @dirs==1?"dir":"dirs"). 
                " with the directory depth of $max_depth...");
    }
    
    my $dh;
    if (!opendir ($dh, $path)) {
        print STDERR "Warning: Can't open dir $path: $!...\n";
        # FOR $unproc_dirs_list_needed ONLY
        push @unproc_dirs, {'dir'=>$path, 'reason'=>$!} if ($unproc_dirs_list_needed);
    }
    else {
	my $dir_entry;
	while ($dir_entry = readdir ($dh)) {
	    next if ($dir_entry=~/^\.\.?$/);    #  "."  ".."

            $path .= '/' unless (substr($path, length($path)-1) eq '/');
            my $dir_entry_path = $path . $dir_entry;

	    next unless (-d $dir_entry_path); #   
	    next if (-l $dir_entry_path); #        
	    &trace ("next directory: $dir_entry...");
	    $curr_depth++;
	    &get_max_dir_depth ($dir_entry_path, 
				$dirs_list_needed,
				$unproc_dirs_list_needed);
	    $curr_depth--;
	}
	closedir $dh;
    }
    &trace ("=== END OF PROCESSING $path DIRECTORY ===");
    if ($curr_depth == 0) {
        #   
	return ($max_depth, \@dirs, \@unproc_dirs);
    }
    else {
        #    
        return;
    }
}
}

sub trace () {
    return unless $opts{'d'};
    my $msg = shift;
    print $msg . "\n";
}

